// Client.java

package purchaseOrderService.client;

import purchaseOrderService.ejb.*;
import java.io.*;
import java.rmi.*;
import java.sql.*;
import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;
import javax.sql.*;
import java.util.StringTokenizer;

import oracle.jdbc.OracleConnection;
import oracle.aurora.jem.*;

//////////////////////////////////////////////////////////////////////

/*
 *  AC4J Client for PurchaseOrder Active EJB
 */
public class Client
{
  static final String DATA_SOURCE_NAME  = "java:comp/env/jdbc/OracleDS";
  static String BEAN_NAME = "java:comp/env/ejb/PurchaseOrderService";
  static final String ACTIVE_EJB_NAME = "JEMPurchaseOrderService";
  static final String METHOD_NAME = "takeOrder";
  public static final String DB_USER    = "JEMUSER";
  public static final String DB_PASSWD  = "JEMPASSWD";

  public static void main(String[] args)
  {
    JEMConnection  ac4j_conn = null;
    JEMSession     ac4j_sess = null;

    if (args.length < 1)
    {
      printUsage();
      return;
    }

    try
    {

      // Get an AC4J Connection and Session
      System.out.println("Getting AC4J Connection and Session...\n");

      // 0. Create a JNDI Context
      Context context = new InitialContext();

      // 1. Look up the DataSource where AC4J Databus resides
      DataSource client_ds = (DataSource) context.lookup(DATA_SOURCE_NAME);
      if (client_ds == null)
      {
        System.out.println("Failed to look up DataSource " +
                           DATA_SOURCE_NAME);
        return;
      }

      // 2. Obtain a JDBC connection to the DataSource
      OracleConnection conn =
        (OracleConnection) client_ds.getConnection(DB_USER, DB_PASSWD);

      // 3. Create an AC4J connection, using the JDBC Connection
      ac4j_conn = new JEMConnection(conn);

      // 4. Create an AC4J Session on the databus
      ac4j_sess = new JEMSession(ac4j_conn);


      // Send a request for a PO or get a response to a PO, depending on
      // command line argument

      if(args[0].equals("reqpo"))
      {
        sendRequest(ac4j_sess, args);
      }
      else
      {
        receiveResponse(ac4j_sess, args);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      System.out.println(e);
      return;
    }
    finally
    {
      try
      {
        // Close AC4J Session and Connection
        if (ac4j_sess != null)
          ac4j_sess.close();
        if (ac4j_conn != null)
          ac4j_conn.close();
      }
      catch(JEMException je)
      {
        je.printStackTrace();
        System.out.println(je);
        return;
      }

    }
  }

  //////////////////////////////////////////////////////////////////////
  static void sendRequest(JEMSession ac4j_sess, String[] args)
    throws  JEMException
  {
    System.out.println("sendRequest: begin");
    System.out.println("  customer name = IID = " + args[1]);
    System.out.println("  credit card number = " + args[2]);
    System.out.println("  item names = " + args[3]);
    System.out.println("  quantities = " + args[4] + "\n");

    JEMHandle activeEJBHandle = null;
    try
    {
      // 0. Create a JNDI-naming context
      Context context = new InitialContext();

      // 1. Look up the handle of the PurchaseOrderService Active EJB
      activeEJBHandle = (JEMHandle)context.lookup(ACTIVE_EJB_NAME);
      if (activeEJBHandle == null)
      {
        System.out.println("Active EJB " + ACTIVE_EJB_NAME + " not found");
        return;
      }
      
      // 2. Prepare input parameters for asynchronous call

      // 2a. Make String and int arrays from itemNames and quantities args
      String[] itemNames  = getStringArrayFromInput(args[3]);
      int[]    quantities = getIntArrayFromInput(args[4]);
      if(itemNames == null || quantities == null)
      {
        System.out.println(args[3] + " OR " + args[4] + " is NULL");
        return;
      }

      if(itemNames.length != quantities.length)
      {
        System.out.println("The number of comma-separated items in " +
                           args[3] +
                           " MUST equal the number of comma-separated " +
                           "quantities in " + args[4]);
        return;
      }

      // 2b. Make a Class array of input parameter types
      Class[] inputClassTypes = new Class[] { String.class,
                                              String.class,
                                              itemNames.getClass(),
                                              quantities.getClass() };

      // 2c. Make an Object array of input parameter values
      Object[] inputParams = new  Object[] { (Object) new  String(args[1]),
                                             (Object) args[2],
                                             (Object) itemNames,
                                             (Object) quantities };

      // 3. Make the asynchronous call
      // IID = customer name (args[1]), AID = null = auto-assigned
      JEMEmitToken request = ac4j_sess.call(args[1],
                                            null,
                                            activeEJBHandle,
                                            METHOD_NAME,
                                            inputClassTypes,
                                            inputParams,
                                            null, 0, 0);

      // 6. Commit the transaction
      ((OracleConnection)ac4j_sess.getJEMConnection().getConnection()).commit();

      // 7. Get the AID auto-assigned to the request just made
      //    Also, Confirm IID (will be value set above - just showing the API)
      //    IID + AID = 'Process Context' of the request
      JEMPortHandle portHandle = request.getPortHandle();
      String iid = portHandle.getIid();
      String aid = portHandle.getAid(); 
      System.out.println("takeOrder request made. Process Context is:");
      System.out.println("  Interaction Identifier (IID) = " + iid);
      System.out.println("  Activation  Identifier (AID) = " + aid);
      System.out.println(
        "\nExecute the following to receive PO response and track status:\n" +
        "  ant -DAID=" + aid + " resppo");

    }
    catch (NamingException e) 
    {
      e.printStackTrace();
      System.out.println(e);
    } 
    catch (SQLException e) 
    {
      e.printStackTrace();
      System.out.println(e);
    } 

  }

  //////////////////////////////////////////////////////////////////////
  static void receiveResponse(JEMSession  ac4j_sess, String[] args)
  {
    System.out.println("receiveResponse: begin");

    JEMHandle activeEJBHandle = null;

    try
    {
      if (ac4j_sess == null)
      {
        System.out.println("null AC4J session");
        return;
      }

      if (args[2].compareTo("undefined") == 0)
      {
        System.out.println("\nError: Activation ID must be passed in :" +
                           "  ant -DAID=<AID value> resppo");
        return;
      }

      // 0. Create a naming context
      Context context = new InitialContext();

      // 1. Look up the handle of the PurchaseOrderService Active EJB
      activeEJBHandle = (JEMHandle)context.lookup(ACTIVE_EJB_NAME);
      if (activeEJBHandle == null)
      {
        System.out.println("Active EJB " + ACTIVE_EJB_NAME + " not found");
        return;
      }

      // 2. Receive Response for the specified Process Context
      //    (IID + AID) plus reaction (takeOrder)
      //    Blocks indefintely, because timeout is 0
      System.out.println("receiveResponse: receiving async response");
      JEMEmitToken resptoken =
        ac4j_sess.receiveReactionResponse(args[1],         // IID
                                          args[2],         // AID
                                          activeEJBHandle, // Active EJB Handle
                                          "takeOrder",     // Reaction/Method
                                          0);              // Timeout (seconds)

      // 3. Retrieve data from Reaction Response
      Object object = resptoken.getReactionResponseObjectInstance();

      // 4. Extract PurchaseOrder Number from data
      Integer poNumber = retrievePONumber(object);
      System.out.println("receiveResponse: response received");
      System.out.println("receiveResponse: Purchase Order number = " +
                         poNumber);

      // 5. Commit the transaction
      ((OracleConnection)ac4j_sess.getJEMConnection().getConnection()).commit();

      // 6. Look up the PurchaseOrder Service Bean
      PurchaseOrderServiceRemote posb = lookupPurchaseOrderServiceBean();

      // 7. Poll the Purchase Order status - synchronously (regular EJB calls)
      //    until the status = shipped
      System.out.println("receiveResponse: polling for PO status...");
      String status = "";
      while(status.compareTo("STATUS_SHIPPED") != 0)
      {
        status = posb.getStatus(poNumber.intValue());
        System.out.println("  status = " + status);
      }

      if (status.compareTo("STATUS_SHIPPED") == 0)
      {
        System.out.println("\nreceiveResponse: Status = SHIPPED. Done.");
      }
    }
    catch (Exception e) 
    {
      e.printStackTrace();
      System.out.println(e);
    }

  }

  //////////////////////////////////////////////////////////////////////
  static Integer retrievePONumber(Object  obj)
  {
    Integer poNumber = null;
    try
    {
      if (obj == null)
      {
        throw new JEMException("retrievePONumber: no object returned");
      }

      if (obj instanceof JEMException)
      {
        JEMException e = (JEMException) obj;
        throw e;
      }
      else if (obj instanceof Throwable)
      {
        Exception e = (Exception) obj;
        throw e;
      }
      else if (obj instanceof java.lang.Integer)
      {
        poNumber = (Integer)  obj;
      }
      else
      {
        throw new JEMException("retrievePONumber: unknown returned Obj");
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      System.out.println(e);
    }
    finally
    {
      return poNumber;
    }
  }

  //////////////////////////////////////////////////////////////////////
  static String[] getStringArrayFromInput(String inString)
    throws JEMException
  {
    String[] stringArray = null;

    StringTokenizer tokenizer = new StringTokenizer(inString, ",");
    if(tokenizer.countTokens() <= 0)
      throw new JEMException("String--" + inString + "--is NULL");
    stringArray = new String[tokenizer.countTokens()];
    for(int i=0; i <= tokenizer.countTokens(); i++)
      stringArray[i] = tokenizer.nextToken();

     return stringArray;
   }

   //////////////////////////////////////////////////////////////////////
   static int[] getIntArrayFromInput(String inString) throws JEMException
   {
     int[] intArray = null;

     StringTokenizer tokenizer = new StringTokenizer(inString, ",");
     if(tokenizer.countTokens() <= 0)
       throw new JEMException("String--" + inString + "--is NULL");
     intArray = new int[tokenizer.countTokens()];
     for(int i=0; i <= tokenizer.countTokens(); i++)
     {
       String s = tokenizer.nextToken();
       Integer intgr = new Integer(s);
       int x = intgr.intValue();
       intArray[i] = x;
     }

    return intArray;
  }

  //////////////////////////////////////////////////////////////////////
  static PurchaseOrderServiceRemote lookupPurchaseOrderServiceBean()
    throws NamingException, RemoteException, Exception
  {
    PurchaseOrderServiceRemote po = null;

    try
    {
      // 0. Create a JNDI-naming context
      Context context = new InitialContext();

      // 1. lookup the home-object
      Object homeObject = context.lookup(BEAN_NAME);

      // 2. Narrow the reference to a AHome.
      PurchaseOrderServiceHome poHome = (PurchaseOrderServiceHome)
        PortableRemoteObject.narrow(homeObject, PurchaseOrderServiceHome.class);

      // 3. find the po
      po = poHome.create();
    }
    catch (Exception e)
    {
      throw e;
    }
    finally
    {
      return po;
    }
  }

  //////////////////////////////////////////////////////////////////////
  static public void printUsage()
  {
    System.out.println("  Client  reqpo <customer name> <credit card number> " +
                       "<item list> <quantities list>");
    System.out.println("   OR");
    System.out.println("  Client  resppo <customer name> <Activation ID>");
  }

}

