#!%ORACLE_HOME%/perl/bin/perl
#
#
# generateLDIF.pl
#
# Copyright (c) 2000, 2002, Oracle Corporation.  All rights reserved.  
#
#    NAME
#      generateLDIF: generates LDIF files from the given templates
#
#    DESCRIPTION
#      This perl script generates LDIF files from  template
#       .sbs files. The idea behind the templates is that
#      the location of the Oracle Context as well as the 
#      user DN is always variable. Also, the LDIF files need
#      to be loaded in a certain sequence. The metafile which
#      is the input to this script contains the list of template
#      files which must be loaded into the directory server and
#      the order in which they appear in the metafile influences
#      the order in which they are loaded. 
#  
#      The variables that this script recognizes in the 
#      templates are as follows:
#         %s_OracleContextDN% : the DN of the OracleContext
#                               this includes the rdn "cn=OracleContext"
#
#         %s_CurrentUserDN% : the DN of the user who is going to 
#                               be loading the LDIF file into the
#                               directory.
#         
#
#####################################################################


use Getopt::Std; # get the standard options package

################################################################
################################################################
## usage function
################################################################
################################################################
## standard usage file 
sub usage 
{
  print qq@
    Usage : $0 -D user_dn  -b context_base -f meta_file

        where

          user_dn: the DN of the user with which you want to load data

          context_base: the DN of the Oracle root context
			e.g. cn=OracleContext.

          meta_file: the file containing the list of LDIF files to be added
          \n@;
  exit();
}


################################################################
################################################################
## variable substitution.logic
################################################################
################################################################
sub do_substitution {
  local($output_file, $user_dn, $context_base, @file_array) = @_;

  # form the context DN correctly based on the base
  $tmp_context_base = $context_base;
  $tmp_context_base =~ s/^ *//;
  if ($tmp_context_base) {
    $context_dn = $context_base;
  }
  else {
    $context_dn = "cn=OracleContext";
  }

  open (LDIF_FILE, ">$output_file")  || die "cannot open file $output_file: $!";
  foreach $file (@file_array) {
    open(SBS_FILE,"$file")  || die "cannot open file $file: $!";
    print LDIF_FILE qq@#################################################################
# Beginning of LDIF from $file   
#################################################################\n@;

SBS_LOOP:
    while( $sbs_line = <SBS_FILE>) {
      $sbs_line =~ s/%s_CurrentUserDN%/$user_dn/g;
      $sbs_line =~ s/%s_OracleContextDN%/$context_dn/g;
      print LDIF_FILE $sbs_line;
    }
    
    print LDIF_FILE qq@#################################################################
# End of LDIF from $file   
#################################################################\n@;
    
    close(SBS_FILE);
  }
  close (LDIF_FILE);

}
  
################################################################
################################################################
## file extraction logic (recursively descend lst files)
################################################################
################################################################
sub get_sbs_files {
  local($lst_file) = @_;
  local(@sbs_files);
  local($meta_line);
  local($template_file);
  local * LST_FILE; ## scope the filehandle locally so that recursion works

  open(LST_FILE , $lst_file)  || die "cannot open file $lst_file : $!";
  
LST_LOOP:
  while($meta_line = <LST_FILE>) {
    chop($meta_line); # get rid of newlines 
    $meta_line =~ s/^ *//; ## get rid of leading spaces on the line
    if($meta_line =~ /^#/) { #ignore comment lines (lines that begin with #)
      # print"comment line: $meta_line \n";
      next LST_LOOP;
    }
    # at this stage we know that we have a valid line 
    if ($meta_line =~ /\./) {
       ($inp_file2,$dummy) = split(/\./, $lst_file);
       $template_file = $inp_file2.".sbs";
       
     # this is old way of getting the ldif template files  
     # ($ldif_template_file, $dummy) = split(/ /, $meta_line);
     # print "ldif template file: $template_file \n";
      if( $template_file =~ /\.lst/) { # if it is a lst file
        print" this is a .lst file, so I am gonna search further in this \n";
        @sbs_files = (@sbs_files, get_sbs_files($template_file));
        print "after search: @sbs_files \n";
      }
      else {
        @sbs_files = (@sbs_files , $template_file);
      }
#push(@ldif_templates, $template_file);
    }
  }
  close(LST_FILE);
#  print "returning: @sbs_files \n";
  return @sbs_files;
}

################################################################
################################################################
## main logic
################################################################
################################################################
# get the options
getopt("Dbf");
if (!$opt_b || !$opt_f ) { # we require 2  options 
 usage;
} 


$user_dn = $opt_D;
$context_base = $opt_b;

$input_file = $opt_f;
($inp_file_name,$dummy) = split(/\./, $input_file);
$output_file = $inp_file_name.".ldif";

#begin process
#print "User DN: $user_dn \n";
#print "Context Base: $context_base \n";
print "Input file: $input_file \n";
print "Output file: $output_file \n";

@ldif_templates = get_sbs_files($input_file);

&do_substitution($output_file, $user_dn, $context_base, @ldif_templates);



